package ac.essex.gp.nodes.imaging.features;

import ac.essex.gp.tree.Node;
import ac.essex.gp.problems.DataStack;
import ac.essex.gp.params.NodeConstraints;
import ac.essex.ooechs.imaging.commons.PixelLoader;

/**
 * A feature (in this context) is a circular area defined by a distance and
 * radius from the central point. It can return either the standard deviation
 * or the mean of a circular area defined by a radius.
 *
 * @author Olly Oechsle, University of Essex, Date: 06-Nov-2007
 * @version 1.0
 */
public abstract class AbstractCircularNode extends Node {

    public double distance, angle;
    public int radius, statistic;

    public AbstractCircularNode() {
        // has four children, defining distance, angle, the size of the circle (radius) and the statistic to return
        super(4);
    }

    public double execute(DataStack data) {

        data.usesImaging = true;

        distance = (child[0].execute(data) * data.getWidth() * 0.5);
        angle = child[1].execute(data) * 2 * Math.PI;
        radius = (int) (child[2].execute(data) * 10);

        if (radius < 1) radius = 1;

        // ensure distance is not too large
        distance = Math.min(distance, (data.getWidth() / 2) - radius);

        if (distance < 0) distance = 0;

        data.value = getData(data.getImage(), 0, 0, data.getWidth(), data.getHeight(), radius);

        return debugger == null? data.value : debugger.record(data.value);

    }

    /**
     * Sub classes must return some value here.
     */
    public abstract double getData(PixelLoader image, int x, int y, int windowWidth, int windowHeight, int radius);

    public int getChildType(int i) {
        return NodeConstraints.PARAMETER;
    }

}
