package ac.essex.gp.nodes.imaging.features;

import ac.essex.gp.tree.Node;
import ac.essex.gp.params.NodeConstraints;
import ac.essex.gp.problems.DataStack;

/**
 * A boxes that are within a particular range from a point
 *
 * @author Olly Oechsle, University of Essex, Date: 06-Jun-2008
 * @version 1.0
 */
public class DissociatedDipole extends Node {

    // defines the bounds
    public static int left = 10;
    public static int right = 10;
    public static int top = 10;
    public static int bottom = 10;

    public DissociatedDipole() {
        // has four children, defining distance, angle, the size of the circle (radius) and the statistic to return
        super(4);
    }

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.RANGE255, NodeConstraints.FEATURE};
    }

    private int dx, dy, width, height;

    private int dx1, dx2, dy1, dy2;

    public double execute(DataStack data) {

        try {

        int totalWidth = (left + right - 1);
        int totalHeight = (top + bottom - 1);

        dx1 = (int) (totalWidth * child[0].execute(data)) - left;
        dy1 = (int) (totalHeight * child[1].execute(data)) - top;

        dx2 = (int) (totalWidth * child[2].execute(data)) - left;
        dy2 = (int) (totalHeight * child[3].execute(data)) - top;

            width = Math.abs(dx2-dx1);
        height = Math.abs(dy2-dy1);

        dx = Math.min(dx1, dx2);
        dy = Math.min(dy1, dy2);

        data.value = data.getImage().getIntegralImage().getMean(data.getX() + dx, data.getY() + dy, data.getX() + dx + width, data.getY() + dy + height);
        return debugger == null? data.value : debugger.record(data.value);

        } catch (ArrayIndexOutOfBoundsException e)  {

            System.out.println("TotalWidth: " + (left + right - 1));
            System.out.println("TotalHeight: " + (top + bottom - 1));
            System.out.println("X: " + data.getX());
        System.out.println("Y: " + data.getY());
        System.out.println("DX: " + dx);
        System.out.println("DY: " + dy);
        System.out.println("DX1: " + dx1);
        System.out.println("DY1: " + dy1);
        System.out.println("DX2: " + dx2);
        System.out.println("DY2: " + dy2);
            System.out.println("Width: " + width);
            System.out.println("Height: " + height);
        System.out.println("DX+WIDTH: " + (dx + width));
        System.out.println("DY+HEIGHT: " + (dy + height));
        System.out.println("X+DX+WIDTH: " + (data.getX()  + dx + width));
        System.out.println("Y+DY+HEIGHT: " + (data.getY() + dy + height));

            throw e;
        }

    }


    /**
     * What would the java be if this node were written in it?
     */
    public String toJava() {
        return "data.getImage().getIntegralImage().getMean(x + " + dx + ", y + " + dy + ", x + "  + (dx + width) + ", y + " + (dy + height) + ")";
    }

    public int getChildType(int i) {
        return NodeConstraints.PARAMETER;
    }

    public String getShortName() {
        return "DD";
    }

}
