package ac.essex.gp.nodes.imaging.features;

import ac.essex.ooechs.imaging.commons.texture.Circle;
import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.gp.params.NodeConstraints;

/**
 * Returns the mean intensity of a region. See feature node for more info.
 *
 * @author Olly Oechsle, University of Essex, Date: 06-Nov-2007
 * @version 1.0
 */
public class SquareMean extends AbstractCircularNode {

    private int dx, dy;
    private int x1, x2, y1, y2;

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.FLOATING_POINT_NUMBER, NodeConstraints.RANGE255, NodeConstraints.FEATURE};
    }

    public double getData(PixelLoader image, int windowLeft, int windowTop, int windowWidth, int windowHeight, int radius) {

        int centerX = windowWidth / 2;
        int centerY = windowHeight / 2;

        dx = (int) (centerX + (distance * Math.sin(angle)));
        dy = (int) (centerY + (distance * Math.cos(angle)));

        x1 = Math.min(windowLeft, windowLeft + dx);
        x2 = Math.max(windowLeft, windowLeft + dx);

        y1 = Math.min(windowTop, windowTop + dy);
        y2 = Math.max(windowTop, windowTop + dy);

        return image.getIntegralImage().getMean(x1, y1, x2, y2);
    }

    public String toJava() {
        // TODO: This may not be right - if dx and dy are negative
        return "image.getIntegralImage().getMean(windowLeft, windowTop, windowLeft + " + dx + ", windowTop" + dy + ")";
    }

    public String getShortName() {
        return "SQm";
    }

}
