package ac.essex.gp.nodes.imaging.texture2;

import ac.essex.gp.tree.Terminal;
import ac.essex.gp.params.NodeConstraints;
import ac.essex.gp.params.RangeTypes;
import ac.essex.gp.problems.DataStack;
import ac.essex.ooechs.imaging.commons.PixelLoader;

/**
 * Gets the mean intensity of pixels on the second horizonal line at the current position on the image
 *
 * @author Olly Oechsle, University of Essex, Date: 05-Jul-2007
 * @version 1.0
 */
public class HLine2Mean extends Terminal {

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.FLOATING_POINT_NUMBER, NodeConstraints.RANGE255, NodeConstraints.FEATURE};
    }

    public double execute(DataStack data) {
        data.usesImaging = true;
        PixelLoader image = data.getImage();
        int x = data.getX();
        int y = data.getY();
        data.value = image.getHLine2().getStatistics(image, x, y).getMean();
        return debugger == null? data.value : debugger.record(data.value);
    }

    public String toJava() {
        return "image.getHLine2().getMean(image, x, y)";
    }

    public String getShortName() {
        return "h2m";
    }

}

