package ac.essex.gp.nodes.imaging.texture2;

import ac.essex.gp.tree.Terminal;
import ac.essex.gp.params.NodeConstraints;
import ac.essex.gp.params.RangeTypes;
import ac.essex.gp.problems.DataStack;
import ac.essex.ooechs.imaging.commons.PixelLoader;

/**
* Gets the standard deviation of the intensity of pixels on a circular perimeter around the current image position.
 *
 * @author Olly Oechsle, University of Essex, Date: 04-Jul-2007
 * @version 1.0
 */
public class Perimeter2Min extends Terminal {

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.INTEGER, NodeConstraints.FEATURE};
    }

    public double execute(DataStack data) {
        data.usesImaging = true;
        PixelLoader image = data.getImage();
        int x = data.getX();
        int y = data.getY();
        data.value = image.getPerimeter2().getStatistics(image, x, y).getMin();
        return debugger == null? data.value : debugger.record(data.value);
    }

    public String toJava() {
        return "image.getPerimeter2().getStatistics(image, x, y).getMin()";
    }

    public String getShortName() {
        return "p2mi";
    }

}
