package ac.essex.gp.nodes.looping;

import ac.essex.gp.tree.Node;
import ac.essex.gp.problems.DataStack;
import ac.essex.gp.params.NodeConstraints;

/**
 * Repeats a piece of code a given number of times.
 *
 * @author Olly Oechsle, University of Essex, Date: 23-Oct-2008
 * @version 1.0
 */
public class Repeat extends Node {

    public Repeat() {
        super(2);
    }

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.VOID};
    }

    /**
     * Any processing the node does takes place here
     */
    public double execute(DataStack data) {
        int iterations = (int) child[0].execute(data);
        for (int i = 0; i < iterations; i++) {
            child[1].execute(data);
        }
        return data.value;
    }

    /**
     * What would the java be if this node were written in it?
     */
    public String toJava() {
        return "for (int i = 0; i < " + child[0].getName() + "; i++)";
    }

    /**
     * Tells what return type each of the node's children should have
     *
     * @param index The index of the child, starting from 0
     */
    public int getChildType(int index) {
        return NodeConstraints.NUMBER;
    }

}
